 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
#  This file creates a suggested solution to "Problem Set 4"    #
#  of the course Microeconometrics, taught at SDU, fall 2012    #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

## NOTE: before running the program you need to change the directory in R to the folder containing the dataset

# Reading libraries into memory
	# The following enables estimation of the MNL model
	library(mlogit)
	# If you have trouble installing the mlogit package, attempt command: install.packages("mlogit")
	# Package MASS contains the 'polr' command making Ordered Probit (and Ordered Logit)
	library(MASS)
	# Agricolae allows easy removal of missing observations
	library(agricolae)


# Loading dataset pension.CSV into memory
	data <- read.table("pension.csv", header=TRUE, sep=",", na.string=".")

	# attaches variable names to dataset, allowing for calling variables individually 
	attach(data)

# 1: Estimating model using OLS
	model.ols <- lm(pctstck ~ choice+age+educ+female+black+married+finc25+finc35+finc50+finc75+finc100+finc101+wealth89+prftshr)
	summary(model.ols)

# 2: Estimating the model using MNL
	# Preparing data for MNL estimation
	data2    <- data.frame(id, pctstck, choice, age, educ, female, black, married, finc25, finc35, finc50, finc75, finc100, finc101, wealth89, prftshr)
	data3    <- delete.na(data2, "greater")
	attach(data3)
	data.mnl <- mlogit.data(data3, choice="pctstck", shape="wide", id="id")

	# Estimating MNL model
	model.mnl <- mlogit(pctstck~0 | choice+age+educ+female+black+married+finc25+finc35+finc50+finc75+finc100+finc101+wealth89+prftshr, data=data.mnl, reflevel="0")	
	summary(model.mnl)

	# Coefficients of each alternative
	length(coef(model.mnl))
	beta.100  <- coef(model.mnl)[seq(1,29, by=2)]
	beta.50   <- coef(model.mnl)[seq(2,30, by=2)]
	
	# Matrix of explanatory variables
	n <- length(data3$id)
	X <- cbind(rep(1,n), data3$choice, data3$age, data3$educ, data3$female, data3$black, data3$married, data3$finc25, data3$finc35, data3$finc50, data3$finc75, data3$finc100, data3$finc101, data3$wealth89, data3$prftshr) 

# 3: Relative risks
	exp(beta.100)
	exp(beta.50)

# 4: Joint significance of income dummies
	# Wald test
	model.res <- mlogit(pctstck~0 | choice+age+educ+female+black+married+wealth89+prftshr, data=data.mnl, reflevel="0")	
	waldtest(model.mnl, model.res, test="Chisq")

	# LR-test
	LR <-	2*(model.mnl$logLik[1]-model.res$logLik[1])
	1-pchisq(LR, df=12)
	# Alternatively, the following command does the same:
	lrtest(model.res, model.mnl)

	# LM test
	scoretest(model.res, model.mnl)

# 5: Predictive power of the model
	# A: Predicted probability of being in each category
	p.0   <- 1/(1+exp(X%*%beta.50)+exp(X%*%beta.100))
	p.50  <- exp(X%*%beta.50)*p.0
	p.100 <- exp(X%*%beta.100)*p.0

	# B: Outcome with highest probability
	predict.0  <- which(p.0>=p.100 & p.0>=p.50)
	predict.50  <- which(p.50>=p.100 & p.50>=p.0)
	predict.100 <- which(p.100>p.50 & p.100>p.0)

	# C: Comparing predicted outcomes with actual outcomes
	prediction <- rep(0,n)
	prediction[predict.50]=50
	prediction[predict.100]=100

	# Actual predictive power of the model
	cat(" The percentage of well predicted outcomes ", sum(prediction==data3$pctstck)/n, "\n")
	cat(" The percentage of pctstck=0 well predicted ",sum(data3$pctstck[predict.0]=="0")/sum(data$pctstck=="0"), "\n")
	cat(" The percentage of pctstck=50 well predicted ",sum(prediction==data3$pctstck & prediction=="50")/sum(data3$pctstck=="50"), "\n")
	cat(" The percentage of pctstck=100 well predicted ",sum(data3$pctstck[predict.100]=="100")/sum(data3$pctstck=="100"), "\n")


# 6: Ordered Probit Model
	data.opm <- data.frame(data3, pctstck.f=as.factor(pctstck))
	model.opm <- polr(pctstck.f ~ choice+age+educ+female+black+married+finc25+finc35+finc50+finc75+finc100+finc101+wealth89+prftshr, data=data.opm, method="probit")	
	summary(model.opm)

# Estimating the marginal effects at the mean (see formula page 506)
	# Extracting coefficients
	polr.coef <- coef(model.opm)
	# Extracting cut-offs
	polr.alfa <- model.opm$zeta
	# Vector of mean values of explanatory variables
	X.opm <- cbind(data3$choice, data3$age, data3$educ, data3$female, data3$black, data3$married, data3$finc25, data3$finc35, data3$finc50, data3$finc75, data3$finc100, data3$finc101, data3$wealth89, data3$prftshr) 
	X.mean <- apply(X.opm, 2, mean)

	# Marginal Effects (at mean) for outcome "0" (NOTE: No distinction made between continuous and discrete variables)
	pctstck0 <- dnorm(polr.alfa[1]-X.mean%*%polr.coef,0,1)
	-polr.coef%*%pctstck0
	# Marginal Effects (at mean) for outcome "50" for variable 'choice'
	polr.coef%*%(dnorm(polr.alfa[1]-X.mean%*%polr.coef,0,1)-dnorm(polr.alfa[2]-X.mean%*%polr.coef,0,1))
	# Marginal Effects (at mean) for outcome "100" for variable 'choice'
	polr.coef%*%dnorm(polr.alfa[2]-X.mean%*%polr.coef,0,1)


# 7: Prediction of Ordered Probit Model
	# Prediction of model
	data.opm <- data.frame(data3, p.opm=predict(model.opm, data3, type="p"))  #Calculates the predicted probabilities for each choice. Skip type and you just get {0,50,100}.
	attach(data.opm)

	# Generating indicator of predicted pctstck-outcome
	predict.opm.0  <- which(p.opm.0>=p.opm.50 & p.opm.0>=p.opm.100)
	predict.opm.50  <- which(p.opm.50>=p.opm.0 & p.opm.50>=p.opm.100)
	predict.opm.100 <- which(p.opm.100>p.opm.50 & p.opm.100>p.opm.0)

	# Comparing predicted with actual outcomes
	prediction.opm  <- rep(0,n)
	prediction.opm[predict.opm.50]=50
	prediction.opm[predict.opm.100]=100

	# Overall prediction of model
	sum(prediction.opm==data.opm$pctstck)/n
	
	# Correct predictions for each outcome
	sum(data.opm$pctstck[-c(predict.opm.50, predict.opm.100)]=="0")/sum(data.opm$pctstck=="0")
	sum(prediction.opm==data.opm$pctstck & prediction.opm=="50")/sum(data.opm$pctstck=="50")
	sum(prediction.opm==data.opm$pctstck & prediction.opm=="100")/sum(data.opm$pctstck=="100")


# 8: Probit model of whether there are any stocks in the pension plan of the individual
	anystck <- as.numeric(pctstck>0)
	model.probit <- glm(anystck ~ choice+age+educ+female+black+married+finc25+finc35+finc50+finc75+finc100+finc101+wealth89+prftshr, data=data3, family=binomial("probit"))	
	summary(model.probit)

